#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

PROGNAME="setroubleshoot"
import gettext
gettext.bindtextdomain(PROGNAME, "/usr/share/locale")
gettext.textdomain(PROGNAME)
try:
    gettext.install(PROGNAME, "/usr/share/locale", 1)
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode

import os
import sys
import pygtk
pygtk.require("2.0")
import gtk, gtk.glade
import pango

import gobject 
import gnome.ui    

from setroubleshoot.config import cfg
from setroubleshoot.signature import *
from setroubleshoot.util import *
from setroubleshoot.log import *
from setroubleshoot.gui_utils import *
import gtkhtml2

#------------------------------------------------------------------------------

NUM_COLUMNS = 6
(PYOBJECT_COLUMN, FILTER_TYPE_COLUMN, DATE_COLUMN, COUNT_COLUMN, CATEGORY_COLUMN,
 SUMMARY_COLUMN) = range(NUM_COLUMNS)

seen_text_attributes = pango.AttrList()

not_seen_text_attributes = pango.AttrList()
not_seen_text_attributes.insert(pango.AttrWeight(pango.WEIGHT_HEAVY, 0, -1))

#------------------------------------------------------------------------------

class BrowserApplet(Applet):

    # --- Initialization ---

    def __init__(self):
        self.cur_siginfo = None
        self.alert_display = None

        # How long an alert must be displayed before it's marked as having been seen
        self.seen_after_displayed_seconds = 3

        self.date_format = '%a %b %e %Y %l:%M %p'

        program_name = _('setroubleshoot browser')
        program_version = '1.0'
        gnome.init(program_name, program_version)
        self.window_name = _("browser")
        glade_file = cfg.get('alert','browser_glade_file')
        self.XML = gtk.glade.XML(glade_file)
        self.wTree = gtk.glade.XML(glade_file, self.window_name)
        self.init_widgets()

        self.seen_icon = self.browser_widget.render_icon(gtk.STOCK_APPLY, gtk.ICON_SIZE_MENU)

        self.hide()


        dic = {'on_browser_delete_event'       : self.on_close,
               'on_open_activate'              : self.on_open_logfile,
               'on_close_activate'             : self.on_close,
               'on_print_activate'             : self.on_print,
               'on_delete_activate'            : self.on_delete,
               'on_save_activate'              : self.on_save,
               'on_report_bug_activate'        : self.on_report_bug,
               'on_about_activate'             : self.on_about,
               } 
        self.wTree.signal_autoconnect(dic) 

    def on_realize(self, widget):
        # position the divider between the top and bottom pane
        vpane_rect = self.vpane_widget.get_allocation()
        self.vpane_widget.set_position(vpane_rect.height/4)

    def init_widgets(self):
        self.window = self.wTree.get_widget(self.window_name)
        self.window.connect_after("realize", self.on_realize)

        self.get_widget("browser")
        self.get_widget("vpane")
        
        self.get_widget("statusbar")
        self.status_context_id = self.statusbar_widget.get_context_id("browser")


        self.get_widget("progressbar")
        self.get_widget("connect_status")

        self.init_list_view('alert_list',
             [ColumnAttr(PYOBJECT_COLUMN,    None,          False, 'pyobject'),
              ColumnAttr(FILTER_TYPE_COLUMN, _('Filter'),   True,  'toggle',   sort_id=FILTER_TYPE_COLUMN, cell_data_func=self.filter_type_cell_data, edit_callback_func=self.on_filter_toggle),
              ColumnAttr(COUNT_COLUMN,       _('Count'),    True,  'int',      sort_id=COUNT_COLUMN,       cell_data_func=self.count_cell_data),
              ColumnAttr(DATE_COLUMN,        _('Date'),     True,  'pyobject', sort_id=DATE_COLUMN,        cell_data_func=self.date_cell_data),
              ColumnAttr(CATEGORY_COLUMN,    _('Category'), True,  'string',   sort_id=CATEGORY_COLUMN,    cell_data_func=self.text_cell_data),
              ColumnAttr(SUMMARY_COLUMN,     _('Summary'),  True,  'string',                               cell_data_func=self.text_cell_data)])

                             
        # alternate row color for easy reading
        self.alert_list_widget.set_rules_hint(True)

        # only permit single selections
        self.alert_list_selection = self.alert_list_widget.get_selection()
        self.alert_list_selection.set_mode(gtk.SELECTION_SINGLE)
        # connect to the selection signal, can't be done with autoconnect
        self.alert_list_selection.connect('changed', self.on_selection_changed)
        # initially select first row
        self.alert_list_selection.select_path((0))

        # FIXME: cruft
        #self.alert_list_widget.connect('button-press-event', self.on_alert_list_button_press)

        self.alert_list_model.connect('sort-column-changed', self.on_sort_column_changed)
        self.alert_list_model.set_sort_func(DATE_COLUMN, self.sort_date_column, DATE_COLUMN)

        # set initial sort order
        self.sort_column_id = DATE_COLUMN
        self.sort_order = gtk.SORT_DESCENDING
        self.alert_list_model.set_sort_column_id(self.sort_column_id, self.sort_order)

        # description
        self.get_widget("description_sw")
        view, doc = self.create_htmlview(self.description_sw_widget)
        doc.connect("link-clicked", self.link_clicked)
        self.description_doc = doc
        self.description_view = view

    def on_report_bug(self, widget):
        # FIXME - Should be specific to the alert
        os.spawnl(os.P_NOWAIT, "/usr/bin/htmlview","htmlview", "http://bugzilla.redhat.com/bugzilla/enter_bug.cgi")
        return True

    def link_clicked(self, arg1, arg2):
        #print "htmlview", "htmlview %s" % arg2
        os.spawnl(os.P_NOWAIT, "/usr/bin/htmlview","htmlview", arg2)
        
    def present(self):
        self.window.present()

    # --- Column Handling ---

    def sort_seen_column(self, model, iter1, iter2, column_index):
        siginfo1   = model.get_value(iter1, PYOBJECT_COLUMN)
        siginfo2   = model.get_value(iter2, PYOBJECT_COLUMN)
        user_data1 = siginfo1.get_user_data(self.server.username)
        user_data2 = siginfo2.get_user_data(self.server.username)

        if user_data1.seen_flag == user_data2.seen_flag:
            return 0
        if user_data1.seen_flag:
            return 1
        else:
            return -1


    def sort_date_column(self, model, iter1, iter2, column_index):
        timestamp1 = model.get_value(iter1, column_index)
        timestamp2 = model.get_value(iter2, column_index)
        if timestamp1 is None or timestamp2 is None:
            return 0
        return timestamp1.cmp(timestamp2)

    def count_cell_data(self, column, cell, model, iter, column_index):
        siginfo = model.get_value(iter, PYOBJECT_COLUMN)
        user_data = siginfo.get_user_data(self.server.username)
        text = model.get_value(iter, column_index)
        if user_data.seen_flag:
            cell.set_property("attributes", seen_text_attributes)
        else:
            cell.set_property("attributes", not_seen_text_attributes)

        # FIXME: this should be a global property, not set each time
        cell.set_property("xalign", 1.0)

        cell.set_property('text', text) 


        return

    def date_cell_data(self, column, cell, model, iter, column_index):
        siginfo = model.get_value(iter, PYOBJECT_COLUMN)
        user_data = siginfo.get_user_data(self.server.username)
        if user_data.seen_flag:
            cell.set_property("attributes", seen_text_attributes)
        else:
            cell.set_property("attributes", not_seen_text_attributes)

        timestamp = model.get_value(iter, column_index)
        if timestamp is None:
            return
        cell.set_property('text', timestamp.format(self.date_format))
        return

    def text_cell_data(self, column, cell, model, iter, column_index):
        siginfo = model.get_value(iter, PYOBJECT_COLUMN)
        user_data = siginfo.get_user_data(self.server.username)
        text = model.get_value(iter, column_index)
        if user_data.seen_flag:
            cell.set_property("attributes", seen_text_attributes)
        else:
            cell.set_property("attributes", not_seen_text_attributes)
        return

    def seen_cell_data(self, column, cell, model, iter, column_index):
        siginfo = model.get_value(iter, PYOBJECT_COLUMN)
        user_data = siginfo.get_user_data(self.server.username)
        if user_data.seen_flag:
            icon = self.seen_icon
        else:
            icon = None
        cell.set_property('pixbuf', icon)
        return

    def filter_type_cell_data(self, column, cell, model, iter, column_index):
        siginfo = model.get_value(iter, PYOBJECT_COLUMN)
        user_data = siginfo.get_user_data(self.server.username)

        if user_data.filter.filter_type == FILTER_NEVER:
            cell.set_property('active', False)
        else:
            cell.set_property('active', True)
        return

    # --- Utilities ---

    def mark_seen(self, siginfo):
        if self.cur_siginfo == siginfo:
            user_data = siginfo.get_user_data(self.server.username)
            user_data.seen_flag = True
            self.alert_list_widget.queue_draw()
            self.server.set_user_data(siginfo.sig, self.server.username, 'seen_flag', True)
        return False

    # --- View Management ---

    def show(self):
        self.window.show()

    def hide(self):
        self.window.hide()

    def __get_table_color(self):
        style = self.window.get_style()
        c = style.bg[gtk.STATE_NORMAL]
        color = "#%.2X%.2X%.2X" % (c.red / 256, c.green / 256, c.blue / 256)
        return color

    def update_alert_view(self, siginfo):
        self.cur_siginfo = siginfo
        self.description_doc.clear()
        self.description_doc.open_stream("text/html")
        self.description_doc.write_stream(siginfo_to_html(siginfo, self.__get_table_color()))
        self.description_doc.close_stream()

        self.description_sw_widget.show_all()
        
        gobject.timeout_add(1000*self.seen_after_displayed_seconds, self.mark_seen, siginfo)

    def clear_alert_view(self):
        self.description_doc.clear()
        self.description_sw_widget.queue_draw()
        #self.description_sw_widget.show_all()

    def init_gui_data(self):
        self.alert_list_model.clear()
        row = preextend_list(NUM_COLUMNS)
        for siginfo in self.sigs.signature_list:
            count = siginfo.report_count
            date = siginfo.last_seen_date
            category = get_text_default(siginfo.category)
            summary = remove_linebreaks(siginfo.solution.summary)
            row[PYOBJECT_COLUMN] = siginfo
            row[COUNT_COLUMN]    = count
            row[DATE_COLUMN]     = date
            row[CATEGORY_COLUMN] = category
            row[SUMMARY_COLUMN]  = summary
            self.alert_list_model.append(row)
        # sort the new data
        self.alert_list_model.set_sort_column_id(self.sort_column_id, self.sort_order)

        # initially select first row
        self.alert_list_selection.select_path((0))
        self.settings = None

    # --- Data Handling ---

    def signatures_updated(self, alert_client, criteria):
        self.load_data()

    def validate_data(self):
        timestamp = TimeStamp()
        for siginfo in self.sigs.signature_list:
            if siginfo.first_seen_date is None:
                siginfo.first_seen_date = timestamp
            if siginfo.last_seen_date is None:
                siginfo.last_seen_date = timestamp

    def load_data(self):
        self.clear_data()
        self.load_data_from_server()

    def load_data_from_server(self):
        criteria = '.'
        self.set_status_message(_('loading data ...'))
        self.progressbar_widget.set_fraction(.5)
        async_rpc = self.server.query_alerts(criteria)
        async_rpc.add_callback(self.query_alerts_callback)
        async_rpc.add_errback(self.query_alerts_error)

    def query_alerts_callback(self, sigs):
        self.sigs = sigs
        self.validate_data()
        self.init_gui_data()
        self.clear_status_message()
        self.progressbar_widget.set_fraction(0)


    def query_alerts_error(self, errno, strerror):
        self.set_status_message("_(load data failed: %s", strerror)
        self.progressbar_widget.set_fraction(0)


    def clear_data(self):
        self.sigs = SEFaultSignatureSet()
        self.init_gui_data()
        self.clear_alert_view()

    # --- Event Handlers ---

    def on_close(self, *args):
        self.hide()
        return True            # return True ==> handled, False ==> propagate

    def on_connection_state_change(self, alert_client, connection_state):
        if connection_state == 'closed':
            self.clear_data()
            self.set_connection_status(False, _('connection closed'))
        elif connection_state == 'open':
            self.set_connection_status(True, _('connected'))
            self.load_data()
        elif connection_state == 'retry':
            self.clear_data()
        elif connection_state == 'hup':
            self.set_connection_status(False, _('connection lost'))
        elif connection_state == 'error':
            self.set_connection_status(False, _('connection failed'))
        elif connection_state == 'invalid':
            pass
        elif connection_state == 'timeout':
            self.set_connection_status(False, _('connection timed out'))
        else:
            log_gui.error("on_connection_state_change: unknown state (%s)", state)


    def on_connection_pending_retry(self, alert_client, seconds_pending):
        if seconds_pending <= 0:
            self.set_connection_status(False, _('retrying connection'))
        else:
            minutes = seconds_pending / 60
            seconds = seconds_pending % 60
            if minutes:
                pending = _("%d minutes %d seconds") % (minutes, seconds)
            else:
                pending = _("%d seconds") % (seconds)
                self.set_connection_status(False, _('retrying connection in %s' % pending))
            

    def on_about(self, widget):
        dlg = self.XML.get_widget("aboutdialog")
        dlg.run()
        dlg.hide()

    def on_save(self, widget):
        dialog = gtk.FileChooserDialog("Save..",
                                       None,
                                       gtk.FILE_CHOOSER_ACTION_SAVE,
                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                        gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        dialog.set_default_response(gtk.RESPONSE_OK)

        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        dialog.add_filter(filter)

        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            model, iter = self.alert_list_selection.get_selected()
            if iter:
                siginfo = model.get_value(iter, PYOBJECT_COLUMN)

            self.save(dialog.get_filename(), siginfo)
        dialog.destroy()

    def save(self, file, siginfo):
        fd=open(file, "w")
        fd.write(siginfo.solution.format_text())
        fd.close()
        
    def on_open_logfile(self, widget):
        title = _("Open logfile for scanning")
        options = ''
        dialog = FileChooserDialog()
        filename = dialog.get_filename(title)
        if filename is None:
            return
        log_gui.debug("on_open_logfile: filename=%s", filename)
        self.set_status_message('opening ... %s' % filename)
        self.progressbar_widget.set_fraction(.25)
        result = self.server.analyze_logfile(filename, options)
        log_gui.debug("analyze_logfile() returned %s", result)
        # FIXME
        if result:
            self.sigs = SEFaultSignatureSet()
            self.sigs.read_xml(result.returnValue, 'sigs')
            self.init_gui_data()

        self.statusbar_widget.pop()


    def draw_page(self, operation, context, page_nr):
        model, iter = self.alert_list_selection.get_selected()
        if iter:
            siginfo = model.get_value(iter, PYOBJECT_COLUMN)
            user_data = siginfo.solution.format_text()
        else:
            return
        cr = context.get_cairo_context()
        layout = context.create_pango_layout()
        layout.set_text(user_data)
        cr.show_layout(layout)

    def on_print(self, widget):
        dialog = gtk.PrintOperation()
        if self.settings != None:
            dialog.set_print_settings(self.settings)
        dialog.connect("draw_page", self.draw_page)
        dialog.set_n_pages(1)

        res = dialog.run(gtk.PRINT_OPERATION_ACTION_PRINT_DIALOG)

        if res == gtk.PRINT_OPERATION_RESULT_APPLY:
            self.settings = dialog.get_print_settings()
      
    def on_selection_changed(self, selection):
        model, iter = selection.get_selected()
        if iter:
            siginfo = model.get_value(iter, PYOBJECT_COLUMN)
            self.update_alert_view(siginfo)

    def on_sort_column_changed(self, treesortable):
        tv_column = self.alert_list_widget.get_column(self.sort_column_id)
        tv_column.set_sort_indicator(False)

        self.sort_column_id, self.sort_order = treesortable.get_sort_column_id()
        tv_column = self.alert_list_widget.get_column(self.sort_column_id)
        tv_column.set_sort_indicator(True)

    def on_filter_toggle(self, cell, path, model):
        iter = model.get_iter_from_string(path)
        siginfo = model.get_value(iter, PYOBJECT_COLUMN)
        user_data = siginfo.get_user_data(self.server.username)

        if user_data.filter.filter_type == FILTER_NEVER:
            user_data.filter.filter_type = FILTER_ALWAYS
        else:
            user_data.filter.filter_type = FILTER_NEVER
        self.server.set_filter(siginfo.sig, self.server.username,
                               user_data.filter.filter_type, '')


    # FIXME: cruft
    def on_alert_list_button_press(self, widget, event):
        # left single button click
        if not (event.type == gtk.gdk.BUTTON_PRESS and event.button == 1):
            return

        path, tv_column, cell_x, cell_y = widget.get_path_at_pos(event.x, event.y)
        row = path[0]
        if tv_column == self.alert_list_widget.get_column(FILTER_TYPE_COLUMN):
            print "toggle filter at row %d" % row

    def verify(self, message):
        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO,
                                gtk.BUTTONS_YES_NO,
                                message)
        dlg.set_position(gtk.WIN_POS_MOUSE)
        dlg.show_all()
        rc = dlg.run()
        dlg.destroy()
        return rc
    
    def on_delete(self, widget):
        model, iter = self.alert_list_selection.get_selected()
        if iter:
            siginfo = model.get_value(iter, PYOBJECT_COLUMN)
            if self.verify(_("Are you sure you want to delete current alert?")) == gtk.RESPONSE_YES:
                self.server.delete_signature(siginfo.sig)

        return True            # return True ==> handled, False ==> propagate

    def set_status_message(self, msg):
        self.statusbar_widget.pop(self.status_context_id)
        self.statusbar_widget.push(self.status_context_id, msg)

    def clear_status_message(self):
        self.statusbar_widget.pop(self.status_context_id)

    def set_connection_status(self, is_connected, msg=None):
        if is_connected:
            self.connect_status_widget.set_from_stock(gtk.STOCK_CONNECT, gtk.ICON_SIZE_SMALL_TOOLBAR)
        else:
            self.connect_status_widget.set_from_stock(gtk.STOCK_DISCONNECT, gtk.ICON_SIZE_SMALL_TOOLBAR)
            
        if msg is not None:
            self.set_status_message(msg)


#-----------------------------------------------------------------------------

# -- Main --

if __name__ == "__main__":
    import getopt

    def usage():
        print '''
        -h help
'''

    try:
        opts, args = getopt.getopt(sys.argv[1:], "h", ["help"])
    except getopt.GetoptError:
        # print help information and exit:
        usage()
        sys.exit(2)

    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit()

    browser_applet = BrowserApplet()
    browser_applet.show()

    gtk.main()
