#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from rhpl.translate import _
import pygtk
pygtk.require("2.0")
import gtk, gtk.glade
import gtkhtml2
import pynotify
import gobject 
import gnome
import os
import datetime
import selinux

import elementtree.ElementTree as ET

from setroubleshoot.config import cfg
from setroubleshoot.signature import *
from setroubleshoot.util import *
from setroubleshoot.log import *

#------------------------------------------------------------------------------

__all__ = ['map_column_types',
           'ColumnAttr',
           'Applet',
           'siginfo_to_html',
           'DateTimeDialog',
           'FileChooserDialog',
          ]


#------------------------------------------------------------------------------

map_column_types = {
    'icon'     : gtk.gdk.Pixbuf,
    'string'   : gobject.TYPE_STRING,
    'int'      : gobject.TYPE_INT,
    'pyobject' : gobject.TYPE_PYOBJECT,
    'toggle'   : gobject.TYPE_BOOLEAN
    }

#------------------------------------------------------------------------------

class ColumnAttr:
    def __init__(self, column, title, visible, type, sort_id=None, cell_data_func=None, edit_callback_func=None):
        self.column = column
        self.title = title
        self.visible = visible
        self.type = type
        self.sort_id = sort_id
        self.cell_data_func = cell_data_func
        self.edit_callback_func = edit_callback_func

#------------------------------------------------------------------------------

class Applet:

    def get_widget(self, name):
        widget_name = '%s_widget' % name
        widget = self.wTree.get_widget(name)
        if widget is None:
            raise ValueError("cannot find %s widget" % name)
        self.__dict__[widget_name] = widget
        return widget

    def init_list_view(self, name, column_attrs=None, rows=None):
        if column_attrs is None:
            column_attrs = [ColumnAttr(0, 'default', True, gobject.TYPE_STRING)]

        column_attrs.sort(lambda a,b: cmp(a.column, b.column))
        column_types = [map_column_types[attr.type] for attr in column_attrs]


        widget = self.get_widget(name)

        model_name  = '%s_model'  % name
        model = gtk.ListStore(*column_types)
        self.__dict__[model_name]  = model

        widget.set_model(model)

        for column_attr in column_attrs:
            if column_attr.type == 'string' or column_attr.type == 'int':
                cell = gtk.CellRendererText()
                tv_column = gtk.TreeViewColumn(column_attr.title, cell, text=column_attr.column)
                tv_column.set_visible(column_attr.visible)
            elif column_attr.type == 'icon':
                cell = gtk.CellRendererPixbuf()
                tv_column = gtk.TreeViewColumn(column_attr.title, cell, pixbuf=column_attr.column)
                tv_column.set_visible(column_attr.visible)
            elif column_attr.type == 'toggle':
                cell = gtk.CellRendererToggle()
                cell.set_property('activatable', True)
                cell.connect( 'toggled', column_attr.edit_callback_func, model)
                tv_column = gtk.TreeViewColumn(column_attr.title, cell)
                tv_column.set_visible(column_attr.visible)
            elif column_attr.type == 'pyobject':
                cell = gtk.CellRendererText()
                tv_column = gtk.TreeViewColumn(column_attr.title, cell)
                tv_column.set_visible(column_attr.visible)
            else:
                raise ValueError('unknown column type = %s' % column_attr.type)
            if column_attr.cell_data_func:
                tv_column.set_cell_data_func(cell, column_attr.cell_data_func, column_attr.column)
            if column_attr.sort_id is not None:
                tv_column.set_sort_column_id(column_attr.sort_id)
                tv_column.set_sort_indicator(False)
            widget.append_column(tv_column)
        
        if rows is not None:
            for row in rows:
                model.append(row)


    def init_text_view(self, name, text=None):
        widget = self.get_widget(name)

        text_buf_name  = '%s_text_buf'  % name
        text_buf = widget.get_buffer()
        self.__dict__[text_buf_name]  = text_buf

        if text is not None:
            text_buf.set_text(text)

    def init_text_entry(self, name, text=None):
        widget = self.get_widget(name)
        
        if text is not None:
            widget.set_text(text)

    def init_combo_box(self, name, column_attrs=None, rows=None):
        if column_attrs is None:
            column_attrs = [ColumnAttr(0, 'default', True, gobject.TYPE_STRING)]

        column_attrs.sort(lambda a,b: cmp(a.column, b.column))
        column_types = [map_column_types[attr.type] for attr in column_attrs]


        widget = self.get_widget(name)

        model_name  = '%s_model'  % name
        model = gtk.ListStore(*column_types)
        self.__dict__[model_name]  = model

        widget.set_model(model)
        widget.clear()

        column_index = 0
        for column_attr in column_attrs:
            if column_attr.visible:
                if column_attr.type == 'string' or column_attr.type == 'int':
                    cell = gtk.CellRendererText()
                    widget.pack_end(cell, True)
                    widget.add_attribute(cell, 'text', column_index)
                elif column_attr.type == 'icon':
                    cell = gtk.CellRendererPixbuf()
                    widget.pack_end(cell, True)
                    widget.add_attribute(cell, 'pixbuf', column_index)
                elif column_attr.type == 'pyobject':
                    cell = gtk.CellRendererText()
                    widget.pack_end(cell, True)
                    widget.add_attribute(cell, 'text', column_index)
                else:
                    raise ValueError('unknown column type = %s' % column_attr.type)
            column_index += 1
        
        
        if rows is not None:
            for row in rows:
                model.append(row)

    def create_htmlview(self, container):
        view = gtkhtml2.View()
        doc = gtkhtml2.Document()
        container.set_hadjustment(view.get_hadjustment())
        container.set_vadjustment(view.get_vadjustment())
        view.set_document(doc)
        container.add(view)

        return (view, doc)

def siginfo_additional_info(siginfo, body):

    table = ET.SubElement(body, "table")
    table.set("cellpadding", "1")

    # This is harder than it should be, but we just want to print
    # out the unique contexts by using a set
    def format_contexts(avc_list, ctx_name):
        s = set()
        for avc in avc_list:
            (rc, trans) = selinux.selinux_raw_to_trans_context(str(getattr(avc, ctx_name)))
            s.add(trans)
        return ", ".join([str(x) for x in s])

    row = ET.SubElement(table, "tr")
    ET.SubElement(row, "td").text = _("Source Context")
    ET.SubElement(row, "td").text = format_contexts(siginfo.avc_list, "scontext")


    row = ET.SubElement(table, "tr")
    td = ET.SubElement(row, "td")
    td.text = _("Target Context")
    ET.SubElement(row, "td").text = format_contexts(siginfo.avc_list, "tcontext")

    # Same as format_contexts - we only want unique _pairs_ of target
    # names and object classes.
    def format_objs(avc_list):
        s = set()
        [s.add((avc.getPath(), avc.tclass)) for avc in avc_list]
        return ", ".join(["%s [ %s ]" % x for x in s])

    row = ET.SubElement(table, "tr")
    ET.SubElement(row, "td").text = _("Target Objects")
    ET.SubElement(row, "td").text = format_objs(siginfo.avc_list)

    def format_rpms(src_rpms, tgt_rpms):
        s = ""
        if len(src_rpms) > 0:
            s = " ".join(["%s [application]" % s for s in src_rpms if s != ""])
        if len(tgt_rpms) > 0:
            return s + " ".join(["%s [target]" % s for s in tgt_rpms if s != ""])
        else:
            return s

    row = ET.SubElement(table, "tr")
    ET.SubElement(row, "td").text = _("Affected RPM Packages")
    ET.SubElement(row, "td").text = format_rpms(siginfo.src_rpm_list, siginfo.tgt_rpm_list)

    row = ET.SubElement(table, "tr")
    ET.SubElement(row, "td").text = _("Policy RPM")
    ET.SubElement(row, "td").text = siginfo.environment.policy_rpm

    row = ET.SubElement(table, "tr")
    ET.SubElement(row, "td").text = _("Selinux Enabled")
    ET.SubElement(row, "td").text = str(siginfo.environment.selinux_enabled)

    row = ET.SubElement(table, "tr")
    ET.SubElement(row, "td").text = _("Policy Type")
    ET.SubElement(row, "td").text = siginfo.environment.policy_type

    row = ET.SubElement(table, "tr")
    ET.SubElement(row, "td").text = _("MLS Enabled")
    ET.SubElement(row, "td").text = str(siginfo.environment.selinux_mls_enabled)

    row = ET.SubElement(table, "tr")
    ET.SubElement(row, "td").text = _("Enforcing Mode")
    ET.SubElement(row, "td").text = siginfo.environment.enforce

    row = ET.SubElement(table, "tr")
    ET.SubElement(row, "td").text = _("Plugin Name")
    ET.SubElement(row, "td").text = siginfo.sig.analysis_id

    row = ET.SubElement(table, "tr")
    ET.SubElement(row, "td").text = _("Host Name")
    ET.SubElement(row, "td").text = siginfo.environment.hostname

    row = ET.SubElement(table, "tr")
    ET.SubElement(row, "td").text = _("Platform")
    ET.SubElement(row, "td").text = siginfo.environment.uname

    ignore = ["a0", "a1", "a2", "a3", "auid", "arch", "success", "syscall"]
    ET.SubElement(body, "p").text = _("Raw Audit Messages")
    ET.SubElement(body, "p").text = "\n".join([avc.to_string(ignore) for avc in siginfo.avc_list])

def siginfo_to_html(siginfo, bg="#FFFFFF"):
    from string import Template
    from xml.sax.saxutils import unescape

    env = siginfo.environment
    summary = remove_linebreaks(siginfo.solution.summary)
    description = remove_linebreaks(siginfo.solution.problem_description)
    fix = remove_linebreaks(siginfo.solution.fix_description)
    fixcmd = remove_linebreaks(siginfo.solution.fix_cmd)

    if siginfo == None:
        return ""

    root = ET.Element("html")
    head = ET.SubElement(root, "head")
    body = ET.SubElement(root, "body")
    body.set("bgcolor", "#ffffff")
    
    table = ET.SubElement(body, "table")
    table.set("width", "100%")
    table.set("cellpadding", "1")

    tr_attribs = { "charoff" : "3" }

    row = ET.SubElement(table, "tr")
    row.set("bgcolor", bg)
    td = ET.SubElement(row, "td", {"valign" : "middle"})
    ET.SubElement(td, "h2").text = _("Summary")
    ET.SubElement(ET.SubElement(row, "tr", tr_attribs), "td").text = summary

    row = ET.SubElement(table, "tr")
    row.set("bgcolor", bg)
    td = ET.SubElement(row, "td", {"valign" : "middle"})
    ET.SubElement(td, "h2").text = _("Detailed Description")
    ET.SubElement(ET.SubElement(row, "tr"), "td").text = description

    row = ET.SubElement(table, "tr")
    row.set("bgcolor", bg)
    td = ET.SubElement(row, "td", {"valign" : "middle"})
    ET.SubElement(td, "h2").text = _("Allowing Access")
    td = ET.SubElement(ET.SubElement(row, "tr"), "td")
    ET.SubElement(td, "p").text = fix
    if fixcmd != "":
        ET.SubElement(td, "br")
        ET.SubElement(td, "p").text = _("The following command will allow this access:")
        ET.SubElement(td, "pre").text = fixcmd
    
    row = ET.SubElement(table, "tr")
    row.set("bgcolor", bg)
    td = ET.SubElement(row, "td", {"valign" : "middle", "colspan" : "2"})
    ET.SubElement(td, "h2").text = _("Additional Information")

    siginfo_additional_info(siginfo, body)
    html = ET.tostring(root)

    return unescape(html)
        
#------------------------------------------------------------------------------

class DateTimeDialog:
    AM = 0
    PM = 1

    def __init__(self):
        program_name = 'setroubleshoot browser'
        program_version = '1.0'
        gnome.init(program_name, program_version)
        self.window_name = "browser"
        self.window_name = "date_time_dialog"
        glade_file = '../gui/setroubleshoot_browser.glade'
        self.wTree = gtk.glade.XML(glade_file, self.window_name)

        self.dialog = self.wTree.get_widget("date_time_dialog")

        self.dt = datetime.datetime.now()

        self.calendar = self.wTree.get_widget("calendar")
        self.hour_spinbutton = self.wTree.get_widget("hour_spinbutton")
        self.minute_spinbutton = self.wTree.get_widget("minute_spinbutton")
        self.am_pm_combobox = self.wTree.get_widget("am_pm_combobox")
        self.cur_datetime_echo = self.wTree.get_widget("cur_datetime_echo")

        self.calendar.connect("month_changed", self.on_update)
        self.calendar.connect("day_selected", self.on_update)
        self.calendar.connect("day_selected_double_click", self.on_update)

        self.hour_spinbutton.connect("value_changed", self.on_update)
        self.minute_spinbutton.connect("value_changed", self.on_update)
        self.am_pm_combobox.connect("changed", self.on_update)

    def on_update(self, widget):
        self.echo_cur_datetime()

    def echo_cur_datetime(self):
        self.get_datetime_from_widgets()
        self.cur_datetime_echo.set_text(self.datetime_echo_string())

    def set_time(self, hour, minute, am_pm):
        date = self.dt.date()
        time = datetime.time(hour, minute)
        self.dt = self.dt.combine(date, time)
        self.update_view()
        return self.dt

    def update_view(self):
        hour = self.dt.hour
        if hour <= 12:
            am_pm = self.AM
        else:
            hour -= 12
            am_pm = self.PM
        minute = self.dt.minute

        year = self.dt.year
        month = self.dt.month - 1
        day = self.dt.day

        self.calendar.select_month(month, year)
        self.calendar.select_day(day)

        self.hour_spinbutton.set_value(hour)
        self.minute_spinbutton.set_value(minute)
        self.am_pm_combobox.set_active(am_pm)
        self.echo_cur_datetime()

    def datetime_echo_string(self):
        if self.am_pm_combobox.get_active() == self.AM:
            am_pm = ' AM'
        else:
            am_pm = ' PM'
        str = self.dt.strftime('%a %b %e %Y %l:%M') + am_pm
        return str

    def get_datetime_from_widgets(self):
        hour = self.hour_spinbutton.get_value_as_int()
        minute = self.minute_spinbutton.get_value_as_int()
        am_pm = self.am_pm_combobox.get_active()
        year, month, day = self.calendar.get_date()
        #print "year=%s month=%s day=%s hour=%s minute=%s" % (year, month, day, hour, minute)
        # calendar widget 1 based, datetime object 0 based
        month += 1
        self.dt = datetime.datetime(year, month, day, hour, minute)
        # adjust for am/pm
        if am_pm == self.PM:
            # pretend 12am == 12pm on the same day
            if hour != 12:
                self.dt += datetime.timedelta(hours=12)
        return self.dt

    def get_datetime(self):
        return self.get_datetime_from_widgets()

    def dialog_get_datetime(self):
        result = None
        self.set_time(12, 0, self.PM)
	if self.dialog.run() == gtk.RESPONSE_OK:
            result = self.get_datetime_from_widgets()
	self.dialog.destroy()
        return result


#------------------------------------------------------------------------------
class FileChooserDialog:
    def __init__(self):
        pass
    
    def get_filename(self, title):
	result = None
	file_open = gtk.FileChooserDialog(title, action=gtk.FILE_CHOOSER_ACTION_OPEN,
            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))

	if file_open.run() == gtk.RESPONSE_OK:
		result = file_open.get_filename()
	file_open.destroy()
        return result
        
#------------------------------------------------------------------------------
