#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from setroubleshoot.rpc import rpc_method, rpc_arg_type, rpc_callback, rpc_signal
from setroubleshoot.signature import *

__all__ = [
    'SETroubleshootServerInterface',
    'SEAlertInterface',
    ]

#-----------------------------------------------------------------------------

class SETroubleshootServerInterface:

    @rpc_method('SETroubleshootServer')
    def analyze_logfile(self, logfile_path, options):
        pass

    @rpc_method('SETroubleshootServer')
    @rpc_arg_type('SETroubleshootServer', SEFaultSignature)
    def delete_signature(self, sig):
        pass

    @rpc_method('SETroubleshootServer')
    def logon(self, type, username, password):
        pass

    @rpc_method('SETroubleshootServer')
    @rpc_arg_type('SETroubleshootServer', SEFaultSignature, str, int, str)
    def set_filter(self, sig, username, filter_type, data):
        pass

    @rpc_method('SETroubleshootServer')
    @rpc_arg_type('SETroubleshootServer', SEFaultSignature, str)
    def evaluate_alert_filter(self, sig, username):
        pass

    @rpc_callback('SETroubleshootServer', 'evaluate_alert_filter')
    def evaluate_alert_filter_callback(self, result):
        pass

    @rpc_method('SETroubleshootServer')
    def query_alerts(self, criteria):
        pass

    @rpc_callback('SETroubleshootServer', 'query_alerts')
    @rpc_arg_type('SETroubleshootServer', SEFaultSignatureSet)
    def query_alerts_callback(self, sigs):
        pass

    @rpc_method('SETroubleshootServer')
    @rpc_arg_type('SETroubleshootServer', SEFaultSignature, str, str, str)
    def set_user_data(self, sig, username, key, value):
        pass

#-----------------------------------------------------------------------------

class SEAlertInterface:

    @rpc_signal('SEAlert')
    @rpc_arg_type('SEAlert', SEFaultSignatureInfo)
    def alert(siginfo):
        pass

    @rpc_signal('SEAlert')
    def signatures_updated(criteria):
        pass


