#
# Authors: Karl MacMillan <kmacmillan@mentalrootkit.com>
#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from setroubleshoot.util import *
from setroubleshoot.Plugin import Plugin
from rhpl.translate import _

class plugin(Plugin):
    summary = _('''
    SELinux prevented the gss daemon from reading unprivileged user temporary files.
    ''')
    
    problem_description = _('''SELinux prevented the gss daemon from
    reading unprivileged user temporary files (e.g., files in /tmp). Allowing this
    access is low risk, but if you have not configured the gss daemon to
    read these files this access request could signal an intrusion
    attempt.''')
    
    fix_description = _('''
    Changing the "$BOOLEAN" boolean to true will allow this access:
    "setsebool -P $BOOLEAN=1."
    ''')
    
    fix_cmd = 'setsebool -P $BOOLEAN=1'

    def __init__(self):
        Plugin.__init__(self, __name__, category=_("Authorization"), bool_name="allow_gssd_read_tmp")

    def analyze(self):
        if self.avc.scontext.type == "gssd_t" \
           and self.avc.targetTypeMatch("user_tmp_t staff_tmp_t") \
           and (self.avc.tclass == "file" or self.avc.tclass == "dir"):
            self.set_alt_path("/tmp")
            return True
        else:
            return False
