#
# Authors: Karl MacMillan <kmacmillan@mentalrootkit.com>
#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from setroubleshoot.util import *
from setroubleshoot.Plugin import Plugin
from rhpl.translate import _

class plugin(Plugin):
    summary = _('''
    SELinux prevented $SOURCE_PATH from accessing the cron spool file.
    ''')

    problem_description = _('''
    SELinux prevented $SOURCE_PATH from accessing the cron spool file.
    This is access is normally needed when using fcron as a cron daemon
    (<a href="http://fcron.free.fr/">http://fcron.free.fr</a>). If you are using fcron you should allow this
    access. Otherwise this access attempt may signal an intrusion attempt.
    ''')

    fix_description = _('''
    Changing the "$BOOLEAN" boolean to true will allow this access:
    "setsebool -P $BOOLEAN=1."
    ''')

    fix_cmd = 'setsebool -P $BOOLEAN=1'

    def __init__(self):
        Plugin.__init__(self, __name__, category=_("CRON"), bool_name="fcron_crond")

    def analyze(self):
        if self.avc.sourceTypeMatch("crond_t") \
               and self.avc.targetTypeMatch("system_cron_spool_t") \
               and self.avc.tclass == "file" \
               and self.avc.accessMatchOne(self.avc.create_file_perms):
            return True
        else:
            return False
