#
# Authors: Dan Walsh <dwalsh@redhat.com>
#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from setroubleshoot.util import *
from setroubleshoot.Plugin import Plugin
from rhpl.translate import _

class plugin(Plugin):
    summary = _('''
    SELinux prevented httpd from $ACCESS http files.
    ''')
    
    problem_description = _('''
    SELinux prevented httpd from $ACCESS files http files.

    Ordinarily httpd is allowed full access to all files labeled with http file
    context.  This machine has a tightened security policy with the $BOOLEAN
    turned off,  This requires explicit labeling of all files.  If a file is
    a cgi script it needs to be labeled with httpd_TYPE_script_exec_t in order
    to be executed.  If it is read only content,it needs to be labeled
    httpd_TYPE_content_t, it is writable content. it needs to be labeled
    httpd_TYPE_script_rw_t or httpd_TYPE_script_ra_t. You can use the
    chcon command to change these context.  Please refer to the man page
    "man httpd_selinux" or 
    <a href="http://fedora.redhat.com/docs/selinux-apache-fc3">FAQ</a>

    ''')
    
    fix_description = _('''
    Changing the "$BOOLEAN" boolean to true will allow this access:
    "setsebool -P $BOOLEAN=1"
    ''')
    
    fix_cmd = 'setsebool -P $BOOLEAN=1'
    
    def __init__(self):
        Plugin.__init__(self, __name__, category=_("Web Server"), bool_name="httpd_unified")

    def analyze(self):
        self.set_priority(75)
        if ( self.avc.tcontext.type == "httpd_t" \
             and (self.avc.tclass == "file" or self.avc.tclass == "dir")
             
             ):
            if self.avc.accessMatchOne("execute execute_no_trans transition"):
                self.set_sub("ACCESS", "executing")
                return True

            if self.avc.accessMatchOne(self.avc.r_file_perms + self.avc.r_dir_perms):
                self.set_sub("ACCESS", "reading")
                return True

            if self.avc.accessMatchOne(self.avc.create_file_perms + self.avc.rw_dir_perms):
                self.set_sub("ACCESS", "reading and writing")
                return True

        return False
