#
# Authors: Dan Walsh <dwalsh@redhat.com>
#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from setroubleshoot.util import *
from setroubleshoot.Plugin import Plugin
from rhpl.translate import _

class plugin(Plugin):
    summary = _('''
    SELinux prevented $SOURCE_PATH from $ACCESS files stored in directories marked with the default context.
    ''')
    
    problem_description = _('''
    SELinux prevented $SOURCE_PATH from $ACCESS files stored in directories marked with the default context.

    $SOURCE_PATH attempted to read one or more files or directories which are
    marked with the default context.  Confined domains usually do not need to
    access these directories, this access attempt could signal an intrusion
    attempt.  If this application and other confined applications need to
    access these files you can either change the file context of the files
    or set a boolean.
    ''')
    
    fix_description = _('''
    Changing the "$BOOLEAN" boolean to true will allow this access:
    "setsebool -P $BOOLEAN=1"
    ''')
    
    fix_cmd = 'setsebool -P $BOOLEAN=1'
    
    def __init__(self):
        Plugin.__init__(self, __name__, bool_name="read_default_t")

    def analyze(self):
        self.set_priority(75)
        if ( self.avc.tcontext.type == "default_t" \
             and (self.avc.tclass == "file" or self.avc.tclass == "dir")):
            if self.avc.accessMatchOne(self.avc.r_file_perms + self.avc.r_dir_perms):
                self.set_sub("ACCESS", "reading")
                return True
            if self.avc.accessMatchOne(self.avc.create_file_perms + self.avc.rw_dir_perms):
                self.set_sub("ACCESS", "reading and writing")
                return True
        return False
