#
# Copyright (C) 2006 Red Hat, Inc. #
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from setroubleshoot.util import *
from setroubleshoot.Plugin import Plugin
from rhpl.translate import _

class plugin(Plugin):
    summary =_('''
    SELinux is preventing the kernel modules from being loaded.
    ''')

    problem_description = _('''
    SELinux has denied kernel module utilities from modifying modify
    kernel modules. This machine is hardened if you did not try to
    manage a kernel module, this probably signals an intrusion.
    ''')

    fix_description = _('''
    If you allow the management of the kernel modules on your machine,
    turn off the $BOOLEAN boolean: "setsebool -P
    $BOOLEAN=0".
    ''')

    fix_cmd = 'setsebool -P $BOOLEAN=0'
    
    def __init__(self):
        Plugin.__init__(self, __name__, category=_("Kernel"), bool_name="secure_mode_insmod")

    def analyze(self):
        if "transition" in self.avc.access and self.avc.tcontext.type == "insmod_exec_t":
            return True
        else:
            return False

