#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from setroubleshoot.util import *
from setroubleshoot.Plugin import Plugin
from rhpl.translate import _

class plugin(Plugin):
    summary =_('''
    SELinux is preventing the SSL tunnel from running as a daemon on $PORTNUMBER.
    ''')

    problem_description = _('''
    SELinux has denied the SSL Tunnel from listening on a network port
    $PORTNUMBER. If the SSL tunnel (stunnel) is not setup to run as a
    daemon, this probably signals an intrusion.
    ''')

    fix_description = _('''
    If you want the SSL Tunnel to run as a daemon you need to turn on
    the $BOOLEAN boolean: "setsebool -P $BOOLEAN=1".  You also need to
    tell SELinux which port SSL Tunnel will be running on.  semanage
    port -a -t stunnel_port_t -p tcp $PORTNUMBER
    ''')

    fix_cmd = 'setsebool -P $BOOLEAN=1'

    def __init__(self):
        Plugin.__init__(self, __name__, category=_("Networking"), bool_name="stunnel_is_daemon")

    def analyze(self):
        if "name_bind" in self.avc.access and self.avc.scontext.type == "stunnel_t":
            return True
        else:
            return False

