#
# Copyright (C) 2006 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

from setroubleshoot.util import *
from setroubleshoot.Plugin import Plugin
from rhpl.translate import _


class plugin(Plugin):
    summary =_('''
    SELinux is preventing xen ($SOURCE_PATH) "$ACCESS" to $TARGET_PATH ($TARGET_TYPE).
    ''')

    problem_description = _('''
    SELinux denied xen access to $TARGET_PATH.
    If this is a XEN image it has to have a file context label of
    xen_image_t. The system is setup to label image files in /xen correctly,
    and this is the preferred location . If you did not intend to use
    $TARGET_PATH as a xen image it could indicate either a bug or it could
    signal a intrusion attempt.
    ''')

    fix_description = _('''
    You can alter the file context by executing chcon -R -t xen_image_t $TARGET_PATH
    ''')

    fix_cmd = 'chcon -R -t xen_image_t $TARGET_PATH'
    
    def __init__(self):
        Plugin.__init__(self, __name__, category=_("File Label"))

    def analyze(self):
        if self.avc.sourceTypeMatch("xend_t xm_t") and \
               (self.avc.tclass == "file" or self.avc.tclass == "dir") and \
               self.avc.accessMatchOne(self.avc.r_file_perms + self.avc.r_dir_perms):
            return True
        else:
            return False

